Module: SafeRequestTimeout::ActiveRecordHook::AdapterInitializer

Defined in:
lib/safe_request_timeout/active_record_hook.rb

Overview

Prepended to AbstractAdapter so the timeout hooks are added to each concrete adapter class the first time it is instantiated. The hooks must go on the concrete class since adapters can override the hooked methods.